@@ -19,9 +19,8 @@ import datetime
 import sys
 
 from absl import logging
-
 import flatbuffers
-from tensorflow.core.framework import graph_debug_info_pb2
+
 from tensorflow.core.protobuf import config_pb2 as _config_pb2
 from tensorflow.core.protobuf import meta_graph_pb2 as _meta_graph_pb2
 from tensorflow.lite.python import conversion_metadata_schema_py_generated as conversion_metadata_fb
@@ -41,9 +40,13 @@ from tensorflow.python.training.saver import export_meta_graph as _export_meta_g
 # The field name of conversion metadata in the flatbuffer file.
 CONVERSION_METADATA_FIELD_NAME = "CONVERSION_METADATA"
 
+# A constant used for mixing the hash.
+HASH_CONST = 0x9e3779b97f4a7800
+
 # Keras functions used by TFLite
 model_input_signature = _tflite_keras_util.model_input_signature
 trace_model_call = _tflite_keras_util.trace_model_call
+get_save_spec = _tflite_keras_util.get_save_spec
 
 # Jax functions used by TFLite
 # pylint: disable=g-import-not-at-top
@@ -173,7 +176,7 @@ def set_tensor_shapes(tensors, shapes):
   """Sets Tensor shape for each tensor if the shape is defined.
 
   Args:
-    tensors: TensorFlow ops.Tensor.
+    tensors: TensorFlow tensor.Tensor.
     shapes: Dict of strings representing input tensor names to list of
       integers representing input shapes (e.g., {"foo": : [1, 16, 16, 3]}).
 
@@ -380,18 +383,8 @@ def convert_debug_info_func(saved_debug_info):
 
   def f(original_nodes):
     """Function to create `GraphDebugInfo` for the given `original_nodes`."""
-    if not saved_debug_info:
-      return None
-
-    output_debug_info = graph_debug_info_pb2.GraphDebugInfo()
-    # All the files are copied over, so the index wouldn't be changed.
-    output_debug_info.files[:] = saved_debug_info.files
-    # We only copy over the debug info for the input nodes
-    for func, node in original_nodes:
-      debug_key = node + "@" + func
-      output_debug_info.traces[debug_key].CopyFrom(
-          saved_debug_info.traces[debug_key])
-    return output_debug_info
+    del original_nodes
+    return saved_debug_info
 
   return f
 
@@ -921,8 +914,15 @@ def _remove_redundant_quantize_ops_per_subgraph(model, subgraph_index,
         for output in signature_def.outputs:
           if output.tensorIndex == op.outputs[0]:
             output.tensorIndex = op.inputs[0]
+      deleted_tensor = requantize_op.inputs[0]
       # Reset the input of the requantize op to the float input
       requantize_op.inputs[0] = op.inputs[0]
+      # Migrate other operator users to output tensor of requantize op
+      for op_user in operators:
+        if deleted_tensor in op_user.inputs and op_user != requantize_op:
+          for idx, input_tensor in enumerate(op_user.inputs):
+            if input_tensor == deleted_tensor:
+              op_user.inputs[idx] = requantize_op.outputs[0]
       operators.remove(op)
 
   # Remove all the quant ops which connect to the output dequant op.
@@ -1012,6 +1012,97 @@ def get_sparsity_modes(model_object):
   return list(result)
 
 
+def get_model_hash(model):
+  """Calculate a 64-bit integer hash for a TensorFlow Lite model based on its structure.
+
+  Args:
+      model: A TensorFlow Lite model object.
+
+  Returns:
+      int: A 64-bit integer hash value representing the model structure.
+  """
+  # TODO(zeekc): Move the hashing implementation to the very end of FlatBuffer
+  # serialization in the c++ layer since not all calls to the converter come via
+  # the Python API.
+  hash_value = 0
+
+  for subgraph in model.subgraphs:
+    if subgraph.operators is not None:
+      hash_value = update_hash_with_primitive_value(
+          hash_value, len(subgraph.operators)
+      )
+
+      for operator in subgraph.operators:
+        if operator.inputs is not None:
+          hash_value = update_hash_with_array(
+              hash_value, operator.inputs
+          )
+
+        if operator.outputs is not None:
+          hash_value = update_hash_with_array(
+              hash_value, operator.outputs
+          )
+
+    if subgraph.tensors is not None:
+      hash_value = update_hash_with_primitive_value(
+          hash_value, len(subgraph.tensors)
+      )
+
+      for tensor in subgraph.tensors:
+        if tensor.name is not None:
+          hash_value = update_hash_with_primitive_value(
+              hash_value, int.from_bytes(tensor.name, "big")
+          )
+
+        if tensor.shape is not None:
+          hash_value = update_hash_with_array(hash_value, tensor.shape)
+
+    if subgraph.inputs is not None:
+      hash_value = update_hash_with_primitive_value(
+          hash_value, len(subgraph.inputs)
+      )
+
+    if subgraph.outputs is not None:
+      hash_value = update_hash_with_primitive_value(
+          hash_value, len(subgraph.outputs)
+      )
+
+  return hash_value
+
+
+def update_hash_with_primitive_value(hash_value, value):
+  """Update the hash value using a primitive value.
+
+  Args:
+      hash_value (int): The current hash value.
+      value: The primitive value to incorporate into the hash.
+
+  Returns:
+      int: The updated hash value.
+  """
+  hash_value = int(value + HASH_CONST +
+                   (hash_value << 10) + (hash_value >> 4))
+
+  # Ensures 64-bit integer range is returned.
+  return (hash_value & ((1 << 63) - 1))
+
+
+def update_hash_with_array(hash_value, int_array):
+  """Update the hash value using a TFLite int array.
+
+  Args:
+      hash_value (int): The current hash value.
+      int_array: A TFLite int array to incorporate into the hash.
+
+  Returns:
+      int: The updated hash value.
+  """
+  if int_array is not None:
+    for i in int_array:
+      hash_value = update_hash_with_primitive_value(hash_value, i)
+  return hash_value
+
+
 def populate_conversion_metadata(model_object, metadata):
   """Add or update conversion metadata to a tflite model.
 
