@@ -229,6 +229,18 @@ class AutotuneOptions(options_lib.OptionsBase):
       docstring="When autotuning is enabled (through `autotune`), determines "
       "the algorithm to use.")
 
+  initial_parallelism = options_lib.create_option(
+      name="initial_parallelism",
+      ty=int,
+      docstring=(
+          "The initial parallelism to use for parallel transformations before"
+          " autotune has a chance to run. A higher value can help with quick"
+          " startup, but may cause the ram_budget to temporarily be exceeded."
+          " Memory-sensitive datasets should consider setting this to `1` to"
+          " avoid running out of memory. Defaults to 16."
+      ),
+  )
+
   def _to_proto(self):
     pb = dataset_options_pb2.AutotuneOptions()
     if self.enabled is not None:
@@ -240,6 +252,8 @@ class AutotuneOptions(options_lib.OptionsBase):
     if self.autotune_algorithm is not None:
       pb.autotune_algorithm = AutotuneAlgorithm._to_proto(  # pylint: disable=protected-access
           self.autotune_algorithm)
+    if self.initial_parallelism is not None:
+      pb.initial_parallelism = self.initial_parallelism
     return pb
 
   def _from_proto(self, pb):
@@ -252,6 +266,8 @@ class AutotuneOptions(options_lib.OptionsBase):
     if pb.WhichOneof("optional_autotune_algorithm") is not None:
       self.autotune_algorithm = AutotuneAlgorithm._from_proto(  # pylint: disable=protected-access
           pb.autotune_algorithm)
+    if pb.WhichOneof("optional_initial_parallelism") is not None:
+      self.initial_parallelism = pb.initial_parallelism
 
   def _set_mutable(self, mutable):
     """Change the mutability value to `mutable` on this options and children."""
@@ -344,6 +360,16 @@ class OptimizationOptions(options_lib.OptionsBase):
       "when the last transformation is a synchronous transformation. If None, "
       "defaults to True.")
 
+  seq_interleave_prefetch = options_lib.create_option(
+      name="seq_interleave_prefetch",
+      ty=bool,
+      docstring=(
+          "Whether to replace parallel interleave using a sequential interleave"
+          " that prefetches elements from its input iterators. If None,"
+          " defaults to False."
+      ),
+  )
+
   map_and_batch_fusion = options_lib.create_option(
       name="map_and_batch_fusion",
       ty=bool,
@@ -389,20 +415,6 @@ class OptimizationOptions(options_lib.OptionsBase):
       docstring="Whether to fuse shuffle and repeat transformations. If None, "
       "defaults to True.")
 
-  warm_start = options_lib.create_option(
-      name="warm_start",
-      ty=bool,
-      docstring=(
-          "Whether to start background threads of asynchronous transformations"
-          " upon iterator creation (as opposed to upon first call to"
-          " `GetNext`). If None, defaults to False.  It should be noted that"
-          " this possibly improves the latency of the initial 'GetNext' call at"
-          " the expense of requiring more memory to hold prefetched elements"
-          " between the time of iterator construction and usage."
-      ),
-      default_factory=lambda: True if test_mode.TEST_MODE else False,
-  )
-
   def _to_proto(self):
     pb = dataset_options_pb2.OptimizationOptions()
     if self.apply_default_optimizations is not None:
@@ -413,6 +425,8 @@ class OptimizationOptions(options_lib.OptionsBase):
       pb.filter_parallelization = self.filter_parallelization
     if self.inject_prefetch is not None:
       pb.inject_prefetch = self.inject_prefetch
+    if self.seq_interleave_prefetch is not None:
+      pb.seq_interleave_prefetch = self.seq_interleave_prefetch
     if self.map_and_batch_fusion is not None:
       pb.map_and_batch_fusion = self.map_and_batch_fusion
     if self.map_and_filter_fusion is not None:
@@ -427,8 +441,6 @@ class OptimizationOptions(options_lib.OptionsBase):
       pb.parallel_batch = self.parallel_batch
     if self.shuffle_and_repeat_fusion is not None:
       pb.shuffle_and_repeat_fusion = self.shuffle_and_repeat_fusion
-    if self.warm_start is not None:
-      pb.warm_start = self.warm_start
     return pb
 
   def _from_proto(self, pb):
@@ -440,6 +452,8 @@ class OptimizationOptions(options_lib.OptionsBase):
       self.filter_parallelization = pb.filter_parallelization
     if pb.WhichOneof("optional_inject_prefetch") is not None:
       self.inject_prefetch = pb.inject_prefetch
+    if pb.WhichOneof("optional_seq_interleave_prefetch") is not None:
+      self.seq_interleave_prefetch = pb.seq_interleave_prefetch
     if pb.WhichOneof("optional_map_and_batch_fusion") is not None:
       self.map_and_batch_fusion = pb.map_and_batch_fusion
     if pb.WhichOneof("optional_map_and_filter_fusion") is not None:
@@ -454,8 +468,6 @@ class OptimizationOptions(options_lib.OptionsBase):
       self.parallel_batch = pb.parallel_batch
     if pb.WhichOneof("optional_shuffle_and_repeat_fusion") is not None:
       self.shuffle_and_repeat_fusion = pb.shuffle_and_repeat_fusion
-    if pb.WhichOneof("optional_warm_start") is not None:
-      self.warm_start = pb.warm_start
 
   def _set_mutable(self, mutable):
     """Change the mutability value to `mutable` on this options and children."""
@@ -607,6 +619,31 @@ class Options(options_lib.OptionsBase):
       ty=ThreadingOptions,
       docstring="DEPRECATED. Use `threading` instead.")
 
+  experimental_warm_start = options_lib.create_option(
+      name="experimental_warm_start",
+      ty=bool,
+      docstring=(
+          "Whether to start background threads of asynchronous transformations "
+          "upon iterator creation, as opposed to during the first call to "
+          "`next()`. Defaults to `False`. "
+          "This improves the latency of the initial 'next()' calls at "
+          "the expense of requiring more memory to hold prefetched elements "
+          "between the time of iterator construction and usage."
+      ),
+      default_factory=lambda: True if test_mode.TEST_MODE else None,
+  )
+
+  dataset_name = options_lib.create_option(
+      name="dataset_name",
+      ty=str,
+      docstring="A name for the dataset, to help in debugging.")
+
+  framework_type = options_lib.create_option(
+      name="framework_type",
+      ty=list,
+      docstring="The list of frameworks that are used to generate this "
+      "pipeline, used for telemetry.")
+
   threading = options_lib.create_option(
       name="threading",
       ty=ThreadingOptions,
@@ -660,6 +697,13 @@ class Options(options_lib.OptionsBase):
       pb.slack = self.experimental_slack
     if self.experimental_symbolic_checkpoint is not None:
       pb.symbolic_checkpoint = self.experimental_symbolic_checkpoint
+    if self.experimental_warm_start is not None:
+      pb.warm_start = self.experimental_warm_start
+    if self.dataset_name is not None:
+      pb.dataset_name = self.dataset_name
+    if self.framework_type:
+      for framework_type in self.framework_type:
+        pb.framework_type.append(framework_type)
     pb.threading_options.CopyFrom(self.threading._to_proto())  # pylint: disable=protected-access
     return pb
 
@@ -677,6 +721,14 @@ class Options(options_lib.OptionsBase):
       self.experimental_slack = pb.slack
     if pb.WhichOneof("optional_symbolic_checkpoint") is not None:
       self.experimental_symbolic_checkpoint = pb.symbolic_checkpoint
+    if pb.WhichOneof("optional_warm_start") is not None:
+      self.experimental_warm_start = pb.warm_start
+    if pb.WhichOneof("optional_dataset_name") is not None:
+      self.dataset_name = pb.dataset_name
+    if pb.framework_type:
+      self.framework_type = []
+      for framework_type in pb.framework_type:
+        self.framework_type.append(framework_type)
     self.threading._from_proto(pb.threading_options)  # pylint: disable=protected-access
 
   def _set_mutable(self, mutable):
