@@ -20,6 +20,7 @@ https://www.tensorflow.org/guide/saved_model#cli_to_inspect_and_execute_savedmod
 """
 
 import argparse
+import platform
 
 import ast
 import os
@@ -43,6 +44,7 @@ from tensorflow.python.framework import tensor_spec
 from tensorflow.python.lib.io import file_io
 from tensorflow.python.platform import tf_logging as logging
 from tensorflow.python.saved_model import load
+from tensorflow.python.saved_model import load_options
 from tensorflow.python.saved_model import loader
 from tensorflow.python.saved_model import save
 from tensorflow.python.saved_model import signature_constants
@@ -183,11 +185,26 @@ _SMCLI_VARIABLES_TO_FEED = flags.DEFINE_string(
     'will NOT be frozen, and their values will be uninitialized in the compiled'
     ' object.')
 
-_SMCLI_TARGET_TRIPLE = flags.DEFINE_string(
-    name='target_triple', default='x86_64-pc-linux',
-    help='Triple identifying a target variation, containing information such as'
-    ' processor architecture, vendor, operating system, and environment. '
-    'Defaults to \'x86_64-pc-linux\'.')
+if platform.machine() == 's390x':
+  _SMCLI_TARGET_TRIPLE = flags.DEFINE_string(
+      name='target_triple',
+      default='',
+      help=(
+          'Triple identifying a target variation, containing information suchas'
+          ' processor architecture, vendor, operating system, and environment.'
+          " Defaults to ''."
+      ),
+  )
+else:
+  _SMCLI_TARGET_TRIPLE = flags.DEFINE_string(
+      name='target_triple',
+      default='x86_64-pc-linux',
+      help=(
+          'Triple identifying a target variation, containing information such'
+          ' as processor architecture, vendor, operating system, and'
+          " environment. Defaults to 'x86_64-pc-linux'."
+      ),
+  )
 
 _SMCLI_TARGET_CPU = flags.DEFINE_string(
     name='target_cpu', default='',
@@ -401,10 +418,20 @@ def _show_defined_functions(saved_model_dir, meta_graphs):
     has_object_graph_def |= meta_graph_def.HasField('object_graph_def')
   if not has_object_graph_def:
     return
-  with ops_lib.Graph().as_default():
-    trackable_object = load.load(saved_model_dir)
-
   print('\nConcrete Functions:', end='')
+  try:
+    with ops_lib.Graph().as_default():
+      trackable_object = load.load(
+          saved_model_dir,
+          options=load_options.LoadOptions(experimental_skip_checkpoint=True))
+  except Exception as e:  # pylint: disable=broad-exception-caught
+    if 'Op type not registered' in str(e):
+      error = 'the existence of custom ops in the SavedModel'
+    else:
+      error = 'unknown reasons'
+    print(f' N/A (could not be listed due to {error})')
+    return
+
   children = list(
       save._AugmentedGraphView(trackable_object)  # pylint: disable=protected-access
       .list_children(trackable_object))
@@ -516,7 +543,7 @@ def _show_all(saved_model_dir):
   saved_model = saved_model_utils.read_saved_model(saved_model_dir)
   for meta_graph_def in sorted(
       saved_model.meta_graphs,
-      key=lambda meta_graph_def: meta_graph_def.meta_info_def.tags,
+      key=lambda meta_graph_def: list(meta_graph_def.meta_info_def.tags),
   ):
     tag_set = meta_graph_def.meta_info_def.tags
     print("\nMetaGraphDef with tag-set: '%s' "
@@ -994,7 +1021,11 @@ def run():
     AttributeError: An error when neither --inputs nor --input_exprs is passed
     to run command.
   """
-  if not _SMCLI_INPUTS.value and not _SMCLI_INPUT_EXPRS.value and not _SMCLI_INPUT_EXAMPLES.value:
+  if (
+      not _SMCLI_INPUTS.value
+      and not _SMCLI_INPUT_EXPRS.value
+      and not _SMCLI_INPUT_EXAMPLES.value
+  ):
     raise AttributeError(
         'At least one of --inputs, --input_exprs or --input_examples must be '
         'required')
